"""
sql templates!
"""

__all__ = ["readTemplates"]

import re
from importlib.resources import files
from .pghook import Hook

red = "\x1b[31m{}\x1b[0m".format
blue= "\x1b[34m{}\x1b[0m".format
# Example: print(f"Hello {red('blue')}")


words = files('funsql').joinpath('data/sql_keywords.txt').read_text().splitlines()


# Example: template.setFormat('{One} {Two}').replaceParam()
# getattr(template.setFormat('{One} {Two}'),'replaceParam')()
class setFormat(str):
    def __init__(self,s):
        self.s = s
    def replaceParam(self,*args,**kwargs):
        s = self.s
        pa = re.findall('{\s?(.+?)\s?}', s)
        argss = list(args)
        param = {}
        for p in pa:
            if p not in param.keys():
                if p in kwargs.keys():
                    param[p] = kwargs.pop(p)
                elif argss:
                    param[p] = argss.pop(0)
                else:
                    param[p] = '{'+p+'}'
#         if kwargs or argss:
#             print(red(str(kwargs)+str(argss))+ ' not used')
        return setQueries(s.format(**param))

# Define methodes available to templates
class setQueries(str):
    def __init__(self,q):
        self.q = q    

    def __call__(self):
        return Hook().get(self.q)

    def __add__(self,other):
        return setQueries(self.q +"\n"+ other)

    def __repr__(self):
        ss = self.q
        # color keywords
        for w in words:
            insensitive_word = re.compile(r"\b" + re.escape(w)+ r"\b", re.IGNORECASE)
            ss = insensitive_word.sub(blue(w), ss)
        # color comments
        ss = re.compile("/\*").sub("\x1b[32m/*", ss)  
        ss = re.compile("\*/").sub("*/\x1b[0m", ss)
        return ss
    
# access templates with "."
# https://stackoverflow.com/questions/2352181/how-to-use-a-dot-to-access-members-of-dictionary
class Map(dict):
    """
    Example:
    m = Map({'first_name': 'Eduardo'}, last_name='Pool', age=24, sports=['Soccer'])
    """
    def __init__(self, *args, **kwargs):
        super(Map, self).__init__(*args, **kwargs)
        for arg in args:
            if isinstance(arg, dict):
                for k, v in arg.items():
                    self[k] = setFormat(v).replaceParam
                    
        if kwargs:
            for k, v in kwargs.items():
                self[k] = setFormat(v).replaceParam

    def __getattr__(self, attr):
        return self.get(attr)

    def __setattr__(self, key, value):
        self.__setitem__(key, value)

    def __setitem__(self, key, value):
        super(Map, self).__setitem__(key, value)
        self.__dict__.update({key: value})

    def __delattr__(self, item):
        self.__delitem__(item)

    def __delitem__(self, key):
        super(Map, self).__delitem__(key)
        del self.__dict__[key]
        
# read templates
def readTemplates(filename):
    templates = {}
    key = None
    with open(filename) as myFile:
        for num, line in enumerate(myFile, 1):
            if line.startswith("-->"):
                key = re.search(r'--> (.*?)\n',line).group(1).strip()
            elif line.startswith("--") or line.strip()=="":
                None
            else:
                if key in templates.keys():
                    templates[key]= templates[key] + line
                else:
                    templates[key] = line
    return Map(templates)

# # add static queries
# # Does not get variable
# s['new'] = template.setQueries('select 1')
# s.new.get()

# # add dynamic queries
# s['dynamic'] = template.setFormat('select {num}').replaceParam
# s.dynamic(3).get()


#!/usr/bin/env python

from setuptools import setup, find_packages

setup(name='funsql',
    version='0.2.1',
    description='highlights sql syntax, and use template files to generate code',
    url='https://db2sql.com/funsql',
    author='Mahdi Jadaliha',
    author_email='jadaliha@gmail.com',
    license='MIT',
    py_modules = ['funsql','funsql.src','funsql.src.template', 'funsql.src.pghook'],
    packages=find_packages(),
    include_package_data=True,
    install_requires=[
        'psycopg2', 'pandas'
    ],
    data_files=[
        ('templates', ['funsql/data/sql_templates.sql']),
        ('keywords', ['funsql/data/sql_keywords.txt'])
    ],
    zip_safe=False)


import psycopg2
from pandas import DataFrame

__all__ = ['Hook']
def singleton(cls):
    instances = {}
    def wrapper(*args, **kwargs):
        if cls not in instances:
          instances[cls] = cls(*args, **kwargs)
        return instances[cls]
    return wrapper

# Here is a class to excute queries on postgres
@singleton
class Hook():
    def __init__(self,db_param={},schema = "pg_catalog"):
        self.__conn_param = db_param
        self.con = psycopg2.connect(**self.__conn_param)
        self.schema = schema
        
    def getConn(self):
        return self.con       
    
    def cur(self,q):
        """
        This function run a query!
        """
        try:
            cur = self.con.cursor()
            cur.execute("rollback")
        except:
            self.con = psycopg2.connect(**self.__conn_param)
            cur = self.con.cursor()
        cur.execute("set search_path to '{}';".format(self.schema))
        cur.execute(q)
        self.con.commit()
        return cur
    
    def run(self,q):
        try:
          dt = self.cur(q).fetchall()
        except:
          dt = []
        return dt
    
    def value(self,q):
        return self.run(q)[0][0]
    
    def get(self,q):
        resoverall = self.cur(q)
        df = DataFrame(resoverall.fetchall())
        df.columns = [desc[0] for desc in resoverall.description]
        return df